/* 
 * 	Copyright (c) 2010 Colorado State University
 * 
 *	Permission is hereby granted, free of charge, to any person
 *	obtaining a copy of this software and associated documentation
 *	files (the "Software"), to deal in the Software without
 *	restriction, including without limitation the rights to use,
 *	copy, modify, merge, publish, distribute, sublicense, and/or
 *	sell copies of the Software, and to permit persons to whom
 *	the Software is furnished to do so, subject to the following
 *	conditions:
 *
 *	The above copyright notice and this permission notice shall be
 *	included in all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *	OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 *	HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 *	WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *	OTHER DEALINGS IN THE SOFTWARE.
 * 
 * 
 *  File: mrtinstance.h
 * 	Authors: He Yan, Dan Massey
 *  Date: Oct 7, 2008 
 */

#ifndef MRTINSTANCE_H_
#define MRTINSTANCE_H_

#include <pthread.h>



/* needed for malloc and free */
#include <stdlib.h>
/* needed for strncpy */
#include <string.h>
/* needed for system error codes */
#include <errno.h>
/* needed for addrinfo struct */
#include <netdb.h>
/* needed for system types such as time_t */
#include <sys/types.h>
/* needed for time function */
#include <time.h>
/* needed for socket operations */
#include <sys/socket.h>
/* needed for pthread related functions */
#include <pthread.h>


#include <stdio.h>
#include <arpa/inet.h>
#include <inttypes.h>
#include <netinet/in.h>

#define MAX_NUM_PEERS 200


// needed for ADDR_MAX_CHARS
#include "../Util/bgpmon_defaults.h"

// needed for QueueWriter
#include "../Queues/queue.h"

/* structure holding mrt information  */
struct MrtStruct
{
	int		id;			// mrt ID number
	char		addr[ADDR_MAX_CHARS];	// mrt's address
	int		port;			// mrt's port
	int		socket;			// mrt's socket for reading
	time_t		connectedTime;		// mrt's connected time
	time_t		lastAction;		// mrt's last action time
	QueueWriter 	qWriter;		// mrt's Peer queue writer 
	int		deleteMrt;		// flag to indicate delete
	int 		labelAction;		// default label action for
	struct MrtStruct *	next;		// pointer to next mrt node
	pthread_t	mrtThreadID;		// thread reference
};
typedef struct MrtStruct MrtNode;

/*-------------------------------------------------------------------------------------- 
 * Purpose: Create a new MrtNode structure.
 * Input:  the mrt ID, address (as string), port, and socket
 * Output: a pointer to the new MrtNode structure 
 *         or NULL if an error occurred.
 * He Yan @ July 22, 2008
 * -------------------------------------------------------------------------------------*/
MrtNode * createMrtNode( long ID, char *addr, int port, int socket, int labelAction );

/*--------------------------------------------------------------------------------------
 * Purpose: The main function of a thread handling one mrt
 * Input:  the mrt node structure
 * Output: none
 * He Yan @ July 22, 2008
 * -------------------------------------------------------------------------------------*/
void * mrtThread( void *arg );


// MRT Peer_Index_Table,   subtype = 1
struct MRT_Index_Table
{
	u_int32_t BGPSrcID;
	u_int16_t ViewNameLen;
	u_int16_t ViewName;
	u_int16_t PeerCount;
	u_int8_t  PeerType;
	u_int32_t PeerBGPID;
	u_int8_t  PeerIP[16];
	u_int32_t PeerAS;
};
typedef struct MRT_Index_Table mrt_index;

// MRT RIB IP
struct MRT_RIB_Table
{
	u_int32_t SeqNumb;
	u_int8_t PrefixLen;
	u_char    Prefix[16];
	u_int16_t EntryCount;
	u_int16_t PeerIndex;
	u_int32_t OrigTime;
	u_int16_t AttrLen;
	u_char    BGPAttr[4096];
}; 
typedef struct MRT_RIB_Table mrt_uni;

// structures for BGP message 
struct MRT_BGP_Message
{
	// IPv4 BGP message
	u_int16_t WDLen;
	u_int16_t AttrLen;
	u_char    mainmsg[4096];
};
typedef struct MRT_BGP_Message mrt_ipv4_message;

struct MRT_BGP_MP_Message
{
	// IPv6 Multiprotocol Reachable NLRI
	u_int16_t WDLen;
	u_int16_t AttrLen;
	u_char    mainmsg[4096];
};
typedef struct MRT_BGP_MP_Message mrt_ipv6_message;





#endif /*MRTINSTANCE_H_*/
