#!/bin/sh
#
# Startup Fedora and RedHat script for BGPmon.
#
# Author: Mikhail Strizhov
# strizhov@cs.colostate.edu
#
#
# chkconfig: 35 85 15     - start or stop process definition within the boot process
# description: BGPmon Monitoring System
# processname: bgpmon
# pidfile: /var/run/bgpmon.pid

# Source function library.      This creates the operating environment for the process to be started
. /etc/rc.d/init.d/functions


BGPMON=bgpmon
BGPMON_EXEC=/usr/local/bin/bgpmon
CONFIG_FILE=/usr/local/etc/bgpmon_config.txt
PIDFILE=/var/run/bgpmon.pid

case "$1" in
  start)
	if [ -f $PIDFILE ]
	then
		echo "$PIDFILE exists, $BGPMON is already running or crashed!"
	else
	        echo -n "Starting $BGPMON: "
        	daemon $BGPMON_EXEC -d -c $CONFIG_FILE -s
		echo
	        touch /var/lock/subsys/$BGPMON
	fi	
        ;;
  stop)

	if [ ! -f $PIDFILE ]
	then
		echo "$PIDFILE does not exist, $BGPMON is not running!"
	else
        	echo -n "Shutting down $BGPMON, please be patient: "
	        killproc $BGPMON_EXEC
	        echo
	        rm -f /var/lock/subsys/$BGPMON
	        rm -f $PIDFILE
		sleep 2
	fi	
        ;;
 *)
        echo "Usage: $0 {start|stop}"
        exit 1
esac

exit 0

